/*
	pao2png
		is based on tiff2png
		(C) 1996 by Willem van Schaik, Singapore
 */

#include  <stdio.h>
#include  <stdlib.h>
#include  "png.h"
#include  "pao2png.h"

static png_struct	*png_ptr;
static png_info		*info_ptr;
static png_color	palette[MAXCOLORS];
static png_text		comm_pair[2];
static int	colors = 0, transparent = -1, planes, depth;
static unsigned char *pool, **row_ptr;


void read_palette(FILE *fp, int cols, int transp)
{
	int i;

	colors = cols;	/* Palette */
	for (i = 0; i < colors; i++) {
		palette[i].red   = (png_byte)getc(fp);
		palette[i].green = (png_byte)getc(fp);
		palette[i].blue  = (png_byte)getc(fp);
	}
	if ((transparent = transp) > colors - 1 && i < MAXCOLORS) {
		colors++;
		palette[i].red   = 255;
		palette[i].green = 255;
		palette[i].blue  = 255;
	}
}


static int get_comment(char *comm)
{
	if (comm == NULL || *comm == 0)
		return 0;

	while (*comm && (*comm == ' ' || *comm == ':')) comm++;
	if (*comm == 0)
		return 0;

	comm_pair[0].compression = -1;	/* uncompressed */
	comm_pair[0].key = "Comment";
	comm_pair[0].text = comm;
	comm_pair[0].text_length = strlen(comm);
	return 1;
}

void open_png(FILE *png, int cols, int rows, int mval, int pnum)
{
	/* start PNG preparation */

	png_ptr = (png_struct *)malloc(sizeof(png_struct));
	info_ptr = (png_info *)malloc(sizeof(png_info));
	if (png_ptr == NULL || info_ptr == NULL) {
		fprintf(stderr, "Error: cannot allocate PNGLIB structures\n");
		exit(1);
	}

	if (pnum < 0 || pnum > 4) {
		fprintf(stderr, "Error: illegal format\n");
		exit(1);
	}
	if (pnum != 0) {
		planes = pnum;
		colors = 0;	/* Not Palette */
	}else
		planes = 1;
	/* allocate space for png-image */
	pool = (unsigned char *)malloc(cols * rows * planes);
	row_ptr = (unsigned char **)malloc(rows * sizeof(unsigned char *));
	if (pool == NULL || row_ptr == NULL) {
		fprintf(stderr, "Error: cannot allocate memory block\n");
		exit(1);
	}

	if (setjmp(png_ptr->jmpbuf)) {
		close_png();
		fprintf (stderr, "Error: setjmp returns error condition\n");
		exit(1);
	}

	png_write_init(png_ptr);
	png_info_init(info_ptr);
	png_init_io(png_ptr, png);
	info_ptr->width = cols;
	info_ptr->height = rows;

	depth = (mval > 16) ? 8 : ((mval > 4) ? 4 : ((mval > 2) ? 2 : 1));
	info_ptr->bit_depth = 8;
	info_ptr->interlace_type = progressive;
	if (verbose) {
		fprintf(stderr, " maxval = %d\n", mval);
		fprintf(stderr, " bit-depth = %d\n", depth);
		fprintf(stderr, " progressive = %s\n",
				progressive ? "ON" : "OFF");
	}

	switch (pnum) {
	case 0:	/* Palette */
		info_ptr->color_type = PNG_COLOR_TYPE_PALETTE;
		if (verbose)
			fprintf(stderr, " color-type = paletted\n");
    
		/* PLTE chunk */
		info_ptr->valid |= PNG_INFO_PLTE;
		info_ptr->palette = palette;
		info_ptr->num_palette = colors;
		if (transparent >= 0) {
			static png_byte buff[2];
			info_ptr->valid |= PNG_INFO_tRNS;
			info_ptr->num_trans = 1; /* only one */
			buff[0] = 0;		/* index of palette */
			info_ptr->trans = buff;
		}
		break;
	case 1: /* Monochrome or Grayscale */
		if (verbose)
			fprintf(stderr, " color-type = grayscale(%d)\n", mval);
		info_ptr->color_type = PNG_COLOR_TYPE_GRAY;
		break;
	case 2: /* Monochrome or Grayscale with ALPHA */
		if (verbose)
		    fprintf(stderr,
		    	" color-type = grayscale(%d) + alpha\n", mval);
		info_ptr->color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
		break;
	case 3: /* RGB 3 planes */
		if (verbose)
			fprintf(stderr, " color-type = truecolor\n");
		info_ptr->color_type = PNG_COLOR_TYPE_RGB;
		break;
	case 4: /* RGB 3 planes with ALPHA */
		if (verbose)
			fprintf(stderr, " color-type = truecolor + alpha\n");
		info_ptr->color_type = PNG_COLOR_TYPE_RGB_ALPHA;
		break;
	}

	/* gAMA chunk */
	if (gamma != -1.0) {
		info_ptr->valid |= PNG_INFO_gAMA;
		info_ptr->gamma = gamma;
		if (verbose)
			fprintf(stderr, " gamma = %f\n", gamma);
	}
}

void close_png(void)
{
	png_write_destroy(png_ptr);
	free((void *)png_ptr);
	free((void *)info_ptr);
	free((void *)pool);
	free((void *)row_ptr);
}

void write_png(FILE *fp, char *comm)
{
	int trflag = 0, npix, total;
	int i, w;
	unsigned char *pp;
	unsigned char tmap[256];
	static const unsigned char elmtwo[4] = { 0, 0x55, 0xaa, 0xff };
	static const unsigned char elmfour[16] = {
		0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
		0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff };

	if (colors > 0 && transparent > 0) {
		png_color temp;
		trflag = 1;
		for (i = 0; i < 256; i++) tmap[i] = i;
		tmap[0] = transparent;
		tmap[transparent] = 0;
		temp = palette[0];
		palette[0] = palette[transparent];
		palette[transparent] = temp;
	}

	png_write_info(png_ptr, info_ptr);
	png_set_packing(png_ptr);

	npix = planes * info_ptr->width;
	total = npix * info_ptr->height;
	pp = pool;
	for (i = 0; i < info_ptr->height; i++) {
		row_ptr[i] = pp;
		pp += npix;
	}
	pp = pool;
	if (trflag) {
		for (i = 0; i < total; i++)
			*pp++ = tmap[ getc(fp) & 0xff ];
	}else if (colors > 0 /* palette index */ || depth == 8) {
		/* depth == 8  or  paletted without trans */
		/* or paletted with trans index 0 */
		for (i = 0; i < total; i++)
			*pp++ = getc(fp);
	}else
		switch (depth) {
		case 1:
			for (i = 0; i < total; i++)
				*pp++ = getc(fp) ? 0xff : 0;
			break;
		case 2:
			for (i = 0; i < total; i++)
				*pp++ = elmtwo[getc(fp) & 0x03];
			break;
		case 4:
			for (i = 0; i < total; i++)
				*pp++ = elmfour[getc(fp) & 0x0f];
			break;
		}
	png_write_image(png_ptr, row_ptr);

	/* Comments */
	if ((w = get_comment(comm)) > 0) {
		info_ptr->num_text = w;
		info_ptr->text = comm_pair;
		if (verbose) {
			for (i = 0; i < w; i++)
				fprintf(stderr, " %s = %s\n",
					comm_pair[i].key, comm_pair[i].text);
		}
	}
	png_write_end(png_ptr, info_ptr);
}
